
//  Questo programma di esempio usa la uart on chip per realizzare una trasmissione
//  e una ricezione di caratteri a polling.  UART1

#include <p18cxxx.h>   /* for TRISA and PORTA declarations */

#pragma CONFIG OSC = INTIO67
#pragma CONFIG WDT = OFF
#pragma CONFIG LVP = OFF

#define RX_VOID  0
#define RX_READY 1
#define RX_ERROR 2


void init_uart(void); // inizializza la uart
void TransmitByte( unsigned char data );  // trasmette un carattere a polling
unsigned char ReceiveByte(unsigned char *ch); // riceve un carattere e ritorna lo stato della uart

void main()
{
 unsigned char ch;

 OSCCON = 0x72;
 
 init_uart(); // 12:9600 baud at 8 MHz

 TransmitByte('U'); TransmitByte('A'); TransmitByte('R'); TransmitByte('T'); TransmitByte('1'); TransmitByte(':'); TransmitByte(0x0d); TransmitByte(0x0a);
 TransmitByte('C'); TransmitByte('I'); TransmitByte('A'); TransmitByte('O'); TransmitByte('!'); TransmitByte(':'); TransmitByte(0x0d); TransmitByte(0x0a);

 while(1) if(ReceiveByte(&ch) == RX_READY) TransmitByte(ch); // forever: echoes back received chars
}

/**************************************************************************/
/* initialize UART */
/**************************************************************************/
void init_uart()
{
 SPBRGH = 0x00; SPBRG = 0x0c; /* sets the baud rate */
 
 // SYNC = 0, BRGH = 0, BRG16 = 0 default values
 
 RCSTAbits.SPEN = 1; // abilita la porta seriale
 RCSTAbits.CREN = 1; // abilita il ricevitore
 TXSTAbits.TXEN = 1; // abilita il trasmettitore
}
/**************************************************************************/
void TransmitByte( unsigned char data )
{
 while (TXSTAbits.TRMT == 0); /* wait for empty transmit buffer */
 TXREG = data; /* start transmission */
}


unsigned char ReceiveByte(unsigned char *ch)
{
	unsigned char status = RCSTA; // legge lo stato della seriale
	
	if(PIR1bits.RCIF == 0) return(RX_VOID);  // se non c' carattere ritorna
	
	if(status & 0x06) // framing or overrun error...
	{
	 RCSTAbits.CREN = 0; RCSTAbits.CREN = 1;  // azzera il flag e riabilita...
	 return(RX_ERROR);
    }
	
	*ch = RCREG;  // RX OK ritorna il carattere ricevuto
	
	return(RX_READY);  // segnala stato OK
}	

